@@ -57,12 +57,18 @@ struct _GnlCompositionPrivate
   GHashTable *objects_hash;
   GMutex *objects_lock;
 
+  GMutex *flushing_lock;
+  gboolean flushing;
+  guint pending_idle;
+
   /* source ghostpad */
   GstPad *ghostpad;
 
   /* current stack, list of GnlObject* */
   GList *current;
 
+  GnlObject *defaultobject;
+
   /*
      current segment seek start/stop time. 
      Reconstruct pipeline ONLY if seeking outside of those values
@@ -83,6 +89,12 @@ struct _GnlCompositionPrivate
   GstPadEventFunction gnl_event_pad_func;
 };
 
+#define OBJECT_IN_ACTIVE_SEGMENT(comp,element) \
+  (((GNL_OBJECT (element)->start >= comp->private->segment_start) &&	\
+    (GNL_OBJECT (element)->start < comp->private->segment_stop)) ||	\
+   ((GNL_OBJECT (element)->stop > comp->private->segment_start) &&	\
+    (GNL_OBJECT (element)->stop <= comp->private->segment_stop)))	\
+
 static void gnl_composition_dispose (GObject * object);
 static void gnl_composition_finalize (GObject * object);
 static void gnl_composition_reset (GnlComposition * comp);
@@ -98,6 +110,9 @@ static GstStateChangeReturn
 gnl_composition_change_state (GstElement * element, GstStateChange transition);
 
 
+static gboolean
+seek_handling (GnlComposition * comp, gboolean initial, gboolean update);
+
 static gboolean
 update_pipeline (GnlComposition * comp, GstClockTime currenttime,
     gboolean initial, gboolean change_state);
@@ -113,8 +128,33 @@ update_pipeline (GnlComposition * comp, GstClockTime currenttime,
 #define COMP_ENTRY(comp, object) \
   (g_hash_table_lookup (comp->private->objects_hash, (gconstpointer) object))
 
-#define COMP_OBJECTS_LOCK(comp) (g_mutex_lock (comp->private->objects_lock))
-#define COMP_OBJECTS_UNLOCK(comp) (g_mutex_unlock (comp->private->objects_lock))
+#define COMP_OBJECTS_LOCK(comp) G_STMT_START {				\
+    GST_LOG_OBJECT (comp, "locking objects_lock from thread %p",		\
+      g_thread_self());							\
+    g_mutex_lock (comp->private->objects_lock);				\
+    GST_LOG_OBJECT (comp, "locked object_lock from thread %p",		\
+		    g_thread_self());					\
+  } G_STMT_END
+
+#define COMP_OBJECTS_UNLOCK(comp) G_STMT_START {			\
+    GST_LOG_OBJECT (comp, "unlocking objects_lock from thread %p",		\
+		    g_thread_self());					\
+    g_mutex_unlock (comp->private->objects_lock);			\
+  } G_STMT_END
+
+#define COMP_FLUSHING_LOCK(comp) G_STMT_START {				\
+    GST_LOG_OBJECT (comp, "locking flushing_lock from thread %p",		\
+      g_thread_self());							\
+    g_mutex_lock (comp->private->flushing_lock);				\
+    GST_LOG_OBJECT (comp, "locked flushing_lock from thread %p",		\
+		    g_thread_self());					\
+  } G_STMT_END
+
+#define COMP_FLUSHING_UNLOCK(comp) G_STMT_START {			\
+    GST_LOG_OBJECT (comp, "unlocking flushing_lock from thread %p",		\
+		    g_thread_self());					\
+    g_mutex_unlock (comp->private->flushing_lock);			\
+  } G_STMT_END
 
 static gboolean gnl_composition_prepare (GnlObject * object);
 
@@ -133,6 +173,7 @@ struct _GnlCompositionEntry
 
   /* handler id for 'no-more-pads' signal */
   gulong nomorepadshandler;
+  gulong padaddedhandler;
   gulong padremovedhandler;
 };
 
@@ -176,20 +217,20 @@ gnl_composition_class_init (GnlCompositionClass * klass)
 
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&gnl_composition_src_template));
-
-/*   gnlobject_class->covers	 = gnl_composition_covers_func; */
-
-/*   klass->nearest_cover	 	 = gnl_composition_nearest_cover_func; */
 }
 
 static void
 hash_value_destroy (GnlCompositionEntry * entry)
 {
-  g_signal_handler_disconnect (entry->object, entry->starthandler);
-  g_signal_handler_disconnect (entry->object, entry->stophandler);
-  g_signal_handler_disconnect (entry->object, entry->priorityhandler);
+  if (entry->starthandler)
+    g_signal_handler_disconnect (entry->object, entry->starthandler);
+  if (entry->stophandler)
+    g_signal_handler_disconnect (entry->object, entry->stophandler);
+  if (entry->priorityhandler)
+    g_signal_handler_disconnect (entry->object, entry->priorityhandler);
   g_signal_handler_disconnect (entry->object, entry->activehandler);
   g_signal_handler_disconnect (entry->object, entry->padremovedhandler);
+  g_signal_handler_disconnect (entry->object, entry->padaddedhandler);
 
   if (entry->nomorepadshandler)
     g_signal_handler_disconnect (entry->object, entry->nomorepadshandler);
@@ -207,8 +248,14 @@ gnl_composition_init (GnlComposition * comp, GnlCompositionClass * klass)
   comp->private->objects_start = NULL;
   comp->private->objects_stop = NULL;
 
+  comp->private->flushing_lock = g_mutex_new ();
+  comp->private->flushing = FALSE;
+  comp->private->pending_idle = 0;
+
   comp->private->segment = gst_segment_new ();
 
+  comp->private->defaultobject = NULL;
+
   comp->private->objects_hash = g_hash_table_new_full
       (g_direct_hash,
       g_direct_equal, NULL, (GDestroyNotify) hash_value_destroy);
@@ -236,6 +283,11 @@ gnl_composition_dispose (GObject * object)
     comp->private->childseek = NULL;
   }
 
+  if (comp->private->current) {
+    g_list_free (comp->private->current);
+    comp->private->current = NULL;
+  }
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -255,6 +307,9 @@ gnl_composition_finalize (GObject * object)
 
   g_mutex_free (comp->private->objects_lock);
   gst_segment_free (comp->private->segment);
+
+  g_mutex_free (comp->private->flushing_lock);
+
   g_free (comp->private);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -271,29 +326,30 @@ unlock_child_state (GstElement * child, GValue * ret, gpointer udata)
 static gboolean
 lock_child_state (GstElement * child, GValue * ret, gpointer udata)
 {
-  GST_DEBUG_OBJECT (child, "unlocking state");
+  GST_DEBUG_OBJECT (child, "locking state");
   gst_element_set_locked_state (child, TRUE);
   return TRUE;
 }
 
-/* static gboolean */
-/* ready_and_lock_child_state (GstElement * child, GValue * ret, gpointer udata) */
-/* { */
-/*   GST_DEBUG_OBJECT (child, */
-/*       "unlocking state, setting to ready, re-locking state"); */
-/*   gst_element_set_locked_state (child, FALSE); */
-/* /\*   gst_element_set_state (child, GST_STATE_READY); *\/ */
-/* /\*   gst_element_set_locked_state (child, TRUE); *\/ */
-/*   return TRUE; */
-/* } */
-
 static void
-gnl_composition_reset (GnlComposition * comp)
+unlock_childs (GnlComposition * comp)
 {
   GstIterator *childs;
   GstIteratorResult res;
   GValue val = { 0 };
 
+  g_value_init (&val, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&val, FALSE);
+  childs = gst_bin_iterate_elements (GST_BIN (comp));
+  res =
+      gst_iterator_fold (childs, (GstIteratorFoldFunction) unlock_child_state,
+      &val, NULL);
+  gst_iterator_free (childs);
+}
+
+static void
+gnl_composition_reset (GnlComposition * comp)
+{
   GST_DEBUG_OBJECT (comp, "resetting");
 
   comp->private->segment_start = GST_CLOCK_TIME_NONE;
@@ -315,14 +371,51 @@ gnl_composition_reset (GnlComposition * comp)
     comp->private->childseek = NULL;
   }
 
-  g_value_init (&val, G_TYPE_BOOLEAN);
-  g_value_set_boolean (&val, FALSE);
-  childs = gst_bin_iterate_elements (GST_BIN (comp));
-  res =
-      gst_iterator_fold (childs, (GstIteratorFoldFunction) unlock_child_state,
-      &val, NULL);
-  gst_iterator_free (childs);
+  unlock_childs(comp);
 
+  COMP_FLUSHING_LOCK (comp);
+  if (comp->private->pending_idle)
+    g_source_remove (comp->private->pending_idle);
+  comp->private->pending_idle = 0;
+  comp->private->flushing = FALSE;
+  COMP_FLUSHING_UNLOCK (comp);
+
+  GST_DEBUG_OBJECT (comp, "Composition now resetted");
+}
+
+static gboolean
+segment_done_main_thread (GnlComposition * comp)
+{
+  /* Set up a non-initial seek on segment_stop */
+  GST_DEBUG_OBJECT (comp, "Setting segment->start to segment_stop:%"GST_TIME_FORMAT,
+		    GST_TIME_ARGS (comp->private->segment_stop));
+  comp->private->segment->start = comp->private->segment_stop;
+
+  seek_handling (comp, TRUE, TRUE);
+
+  if (!comp->private->current) {
+    /* If we're at the end, post SEGMENT_DONE, or push EOS */
+    GST_DEBUG_OBJECT (comp, "Nothing else to play");
+
+    if (!(comp->private->segment->flags & GST_SEEK_FLAG_SEGMENT)
+	&& comp->private->ghostpad)
+      gst_pad_push_event (comp->private->ghostpad, gst_event_new_eos ());
+    else if (comp->private->segment->flags & GST_SEEK_FLAG_SEGMENT) {
+      gint64 epos;
+      
+      if (GST_CLOCK_TIME_IS_VALID (comp->private->segment->stop))
+	epos =
+	  (MIN (comp->private->segment->stop, GNL_OBJECT (comp)->stop));
+      else
+	epos = (GNL_OBJECT (comp)->stop);
+      
+      GST_BIN_CLASS (parent_class)->handle_message
+	(GST_BIN (comp),
+	 gst_message_new_segment_done (GST_OBJECT (comp),
+                    comp->private->segment->format, epos));
+    }
+  }
+  return FALSE;
 }
 
 static void
@@ -331,77 +424,42 @@ gnl_composition_handle_message (GstBin * bin, GstMessage * message)
   GnlComposition *comp = GNL_COMPOSITION (bin);
   gboolean dropit = FALSE;
 
-  GST_DEBUG_OBJECT (comp, "message:%s",
-      gst_message_type_get_name (GST_MESSAGE_TYPE (message)));
+  GST_DEBUG_OBJECT (comp, "message:%s from %s",
+		    gst_message_type_get_name (GST_MESSAGE_TYPE (message)),
+		    GST_MESSAGE_SRC (message) ? GST_ELEMENT_NAME (GST_MESSAGE_SRC (message)) : "UNKNOWN");
 
   switch (GST_MESSAGE_TYPE (message)) {
+  case GST_MESSAGE_SEGMENT_START: {
+    COMP_FLUSHING_LOCK (comp);
+    if (comp->private->pending_idle) {
+      GST_DEBUG_OBJECT (comp, "removing pending seek for main thread");
+      g_source_remove (comp->private->pending_idle);
+    }
+    comp->private->pending_idle = 0;
+    comp->private->flushing = FALSE;
+    COMP_FLUSHING_UNLOCK (comp);
+    dropit = TRUE;
+    break;
+  }
     case GST_MESSAGE_SEGMENT_DONE:{
-      gint64 pos;
-      GstFormat format;
-
-      /* reorganize pipeline */
-
-      gst_message_parse_segment_done (message, &format, &pos);
-      if (format != GST_FORMAT_TIME) {
-        /* this can happen when filesrc emits a segment-done in BYTES */
-        GST_WARNING_OBJECT (comp,
-            "Got a SEGMENT_DONE_MESSAGE with a format different from GST_FORMAT_TIME");
+      COMP_FLUSHING_LOCK (comp);
+      if (comp->private->flushing) {
+	GST_DEBUG_OBJECT (comp, "flushing, bailing out");
+	COMP_FLUSHING_UNLOCK (comp);
+	dropit = TRUE;
+	break;
       }
-      GST_DEBUG_OBJECT (comp,
-          "Saw a SEGMENT_DONE message [%" GST_TIME_FORMAT
-          "] from %s (comp->private->segment[%" GST_TIME_FORMAT "--%"
-          GST_TIME_FORMAT "])",
-          GST_TIME_ARGS (pos),
-          GST_OBJECT_NAME (GST_MESSAGE_SRC (message)),
-          GST_TIME_ARGS (comp->private->segment_start),
-          GST_TIME_ARGS (comp->private->segment_stop));
+      COMP_FLUSHING_UNLOCK (comp);
 
-      if ((pos <= comp->private->segment_stop)
-          && (pos > comp->private->segment_start)) {
-        /* If we are switching from one object to another (rather than brutal seek), we
-           want to do a flush-less update */
-        gboolean initial = (pos == comp->private->segment_stop) ? TRUE : FALSE;
 
-        GST_DEBUG_OBJECT (comp,
-            "position within current segment, updating pipeline");
-        update_pipeline (comp, (GstClockTime) comp->private->segment_stop,
-            initial, TRUE);
-
-        if (!(comp->private->current)) {
-          GST_DEBUG_OBJECT (comp, "Nothing else to play");
-
-          /*
-             We drop all segments and only emit SEGMENT_DONE if segment->flags had segment
-             and we've finished.
-           */
-          gst_message_unref (message);
-
-          if (!(comp->private->segment->flags & GST_SEEK_FLAG_SEGMENT)
-              && comp->private->ghostpad)
-            gst_pad_push_event (comp->private->ghostpad, gst_event_new_eos ());
-          else if (comp->private->segment->flags & GST_SEEK_FLAG_SEGMENT) {
-            gint64 epos;
-
-            if (GST_CLOCK_TIME_IS_VALID (comp->private->segment->stop))
-              epos =
-                  (MIN (comp->private->segment->stop, GNL_OBJECT (comp)->stop));
-            else
-              epos = (GNL_OBJECT (comp)->stop);
-
-            GST_BIN_CLASS (parent_class)->handle_message
-                (bin, gst_message_new_segment_done (GST_OBJECT (comp),
-                    comp->private->segment->format, epos));
-          }
-          GST_DEBUG_OBJECT (comp, "END of Nothing else to play");
-        }
-
-        return;
-      } else {
-        GST_DEBUG_OBJECT (comp,
-            "position outside current segment, discarding message");
-        dropit = TRUE;
+      GST_DEBUG_OBJECT (comp, "Adding segment_done handling to main thread");
+      if (comp->private->pending_idle) {
+	GST_WARNING_OBJECT (comp, "There was already a pending segment_done in main thread !");
+	g_source_remove (comp->private->pending_idle);
       }
+      comp->private->pending_idle = g_idle_add ((GSourceFunc) segment_done_main_thread, (gpointer)comp);
 
+      dropit = TRUE;
       break;
     }
     default:
@@ -458,7 +516,7 @@ get_new_seek_event (GnlComposition * comp, gboolean initial)
   if (!(initial))
     flags = comp->private->segment->flags;
   else
-    flags = GST_SEEK_FLAG_SEGMENT | GST_SEEK_FLAG_ACCURATE;
+    flags = GST_SEEK_FLAG_SEGMENT | GST_SEEK_FLAG_ACCURATE | GST_SEEK_FLAG_FLUSH;
 
   start = MAX (comp->private->segment->start, comp->private->segment_start);
   stop = GST_CLOCK_TIME_IS_VALID (comp->private->segment->stop)
@@ -473,6 +531,28 @@ get_new_seek_event (GnlComposition * comp, gboolean initial)
       GST_SEEK_TYPE_SET, stop);
 }
 
+/*
+  Figures out if pipeline needs updating. Updates it and sends the seek event.
+  can be called by user_seek or segment_done
+*/
+
+static gboolean
+seek_handling (GnlComposition * comp, gboolean initial, gboolean update)
+{
+  GST_DEBUG_OBJECT (comp, "initial:%d, update:%d", initial, update);
+
+  COMP_FLUSHING_LOCK (comp);
+  GST_DEBUG_OBJECT (comp, "Setting flushing to TRUE");
+  comp->private->flushing = TRUE;
+  COMP_FLUSHING_UNLOCK (comp);
+
+  if (update || have_to_update_pipeline (comp)) {
+    update_pipeline (comp, comp->private->segment->start, initial, TRUE);
+  }
+
+  return TRUE;
+}
+
 static void
 handle_seek_event (GnlComposition * comp, GstEvent * event)
 {
@@ -502,10 +582,7 @@ handle_seek_event (GnlComposition * comp, GstEvent * event)
   comp->private->segment->stop = MIN (comp->private->segment->stop,
       GNL_OBJECT (comp)->stop);
 
-  /* Check if we need to update the pipeline */
-  if (have_to_update_pipeline (comp)) {
-    update_pipeline (comp, comp->private->segment->start, FALSE, TRUE);
-  };
+  seek_handling (comp, TRUE, FALSE);
 }
 
 static gboolean
@@ -527,13 +604,26 @@ gnl_composition_event_handler (GstPad * ghostpad, GstEvent * event)
 
   if (res) {
     GST_DEBUG_OBJECT (comp, "About to call gnl_event_pad_func()");
+    COMP_OBJECTS_LOCK (comp);
     res = comp->private->gnl_event_pad_func (ghostpad, event);
+    COMP_OBJECTS_UNLOCK (comp);
     GST_DEBUG_OBJECT (comp, "Done calling gnl_event_pad_func() %d", res);
   }
   gst_object_unref (comp);
   return res;
 }
 
+static void
+pad_blocked (GstPad * pad, gboolean blocked, GnlComposition * comp)
+{
+  GST_DEBUG_OBJECT (comp, "Pad : %s:%s , blocked:%d",
+		    GST_DEBUG_PAD_NAME (pad), blocked);
+}
+
+/* gnl_composition_ghost_pad_set_target:
+ * target: The target #GstPad. The refcount will be decremented (given to the ghostpad).
+ */
+
 static void
 gnl_composition_ghost_pad_set_target (GnlComposition * comp, GstPad * target)
 {
@@ -553,26 +643,26 @@ gnl_composition_ghost_pad_set_target (GnlComposition * comp, GstPad * target)
       GST_DEBUG_OBJECT (comp,
           "Target of ghostpad is the same as existing one, not changing");
       gst_object_unref (ptarget);
-      gst_object_ref (target);
       return;
     }
 
     GST_DEBUG_OBJECT (comp, "Previous target was %s:%s, blocking that pad",
         GST_DEBUG_PAD_NAME (ptarget));
+    gst_pad_set_blocked_async (ptarget, TRUE, (GstPadBlockCallback) pad_blocked, comp);
 
     gst_object_unref (ptarget);
   }
 
   gnl_object_ghost_pad_set_target (GNL_OBJECT (comp),
       comp->private->ghostpad, target);
-  GST_DEBUG_OBJECT (comp->private->ghostpad, "About to replace event_pad_func");
-  comp->private->gnl_event_pad_func =
-      GST_PAD_EVENTFUNC (comp->private->ghostpad);
-  gst_pad_set_event_function (comp->private->ghostpad,
-      GST_DEBUG_FUNCPTR (gnl_composition_event_handler));
-  GST_DEBUG_OBJECT (comp->private->ghostpad, "eventfunc is now %s",
-      GST_DEBUG_FUNCPTR_NAME (GST_PAD_EVENTFUNC (comp->private->ghostpad)));
   if (!(hadghost)) {
+    GST_DEBUG_OBJECT (comp->private->ghostpad, "About to replace event_pad_func");
+    comp->private->gnl_event_pad_func =
+      GST_PAD_EVENTFUNC (comp->private->ghostpad);
+    gst_pad_set_event_function (comp->private->ghostpad,
+				GST_DEBUG_FUNCPTR (gnl_composition_event_handler));
+    GST_DEBUG_OBJECT (comp->private->ghostpad, "eventfunc is now %s",
+      GST_DEBUG_FUNCPTR_NAME (GST_PAD_EVENTFUNC (comp->private->ghostpad)));
     if (!(gst_element_add_pad (GST_ELEMENT (comp), comp->private->ghostpad)))
       GST_WARNING ("Couldn't add the ghostpad");
     else
@@ -581,6 +671,44 @@ gnl_composition_ghost_pad_set_target (GnlComposition * comp, GstPad * target)
   GST_DEBUG_OBJECT (comp, "END");
 }
 
+static GstClockTime
+next_stop_in_region_above_priority (GnlComposition * composition,
+				    GstClockTime start,
+				    GstClockTime stop, guint priority)
+{
+  GList * tmp = composition->private->objects_start;
+  GnlObject * object;
+  GstClockTime res = stop;
+
+  GST_DEBUG_OBJECT (composition, "start: %" GST_TIME_FORMAT " stop: %"
+		    GST_TIME_FORMAT " priority:%d",
+		    GST_TIME_ARGS (start),
+		    GST_TIME_ARGS (stop),
+		    priority);
+
+  /* Find first object with priority < priority, and start < start_position <= stop */
+  for (; tmp ; tmp = g_list_next (tmp)) {
+    object = tmp->data;
+
+    if (object->priority >= priority)
+      continue;
+
+    if (object->stop <= start)
+      continue;
+
+    if (object->start > stop)
+      break;
+    
+    res = object->start;
+    GST_DEBUG_OBJECT (composition, "Found %s [prio:%d] at %" GST_TIME_FORMAT,
+		      GST_OBJECT_NAME (object),
+		      object->priority,
+		      GST_TIME_ARGS (object->start));
+    break;
+  }
+  return res;
+}
+
 /**
  * get_stack_list:
  * @comp: The #GnlComposition
@@ -623,21 +751,23 @@ get_stack_list (GnlComposition * comp, GstClockTime timestamp,
         object->priority);
 
     if (object->start <= timestamp) {
-      if (object->stop <= timestamp) {
-        if (stack) {
-          GST_LOG_OBJECT (comp, "too far, stopping iteration");
-          break;
-        }
-      } else if ((object->priority >= priority)
-          && ((!activeonly) || (object->active))) {
+      if ((object->stop > timestamp) &&
+	  (object->priority >= priority) &&
+	  ((!activeonly) || (object->active))) {
         GST_LOG_OBJECT (comp, "adding %s: sorted to the stack",
             GST_OBJECT_NAME (object));
         stack = g_list_insert_sorted (stack, object,
             (GCompareFunc) priority_comp);
       }
+    } else {
+      GST_LOG_OBJECT (comp, "too far, stopping iteration");
+      break;
     }
   }
 
+  if ((timestamp < GNL_OBJECT(comp)->stop) && comp->private->defaultobject)
+    stack = g_list_append(stack, comp->private->defaultobject);
+
   return stack;
 }
 
@@ -658,11 +788,14 @@ get_clean_toplevel_stack (GnlComposition * comp, GstClockTime * timestamp,
   GList *ret = NULL;
   gint size = 1;
   GstClockTime stop = G_MAXUINT64;
+  guint top_priority = -1;
 
   GST_DEBUG_OBJECT (comp, "timestamp:%" GST_TIME_FORMAT,
       GST_TIME_ARGS (*timestamp));
 
   stack = get_stack_list (comp, *timestamp, 0, TRUE);
+
+  /* Case for gaps, therefore no objects at specified *timestamp */
   if (!stack) {
     GnlObject *object = NULL;
 
@@ -683,9 +816,12 @@ get_clean_toplevel_stack (GnlComposition * comp, GstClockTime * timestamp,
       stack = get_stack_list (comp, *timestamp, 0, TRUE);
     }
   }
+
   for (tmp = stack; tmp && size; tmp = g_list_next (tmp), size--) {
     GnlObject *object = tmp->data;
 
+    if (top_priority == -1)
+      top_priority = object->priority;
     ret = g_list_append (ret, object);
     if (stop > object->stop)
       stop = object->stop;
@@ -693,6 +829,9 @@ get_clean_toplevel_stack (GnlComposition * comp, GstClockTime * timestamp,
     /* size += number of input for operation */
   }
 
+  /* Figure out if there's anything blocking us with smaller priority */
+  stop = next_stop_in_region_above_priority (comp, *timestamp, stop, top_priority);
+  
   g_list_free (stack);
 
   if (stop_time) {
@@ -802,21 +941,6 @@ gnl_composition_change_state (GstElement * element, GstStateChange transition)
     return ret;
 
   switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:{
-      GstIterator *childs;
-      GstIteratorResult res;
-      GValue val = { 0 };
-
-      /* state-lock all elements */
-      GST_DEBUG_OBJECT (comp, "Locking all childs");
-      g_value_init (&val, G_TYPE_BOOLEAN);
-      g_value_set_boolean (&val, FALSE);
-      childs = gst_bin_iterate_elements (GST_BIN (comp));
-      res = gst_iterator_fold (childs,
-          (GstIteratorFoldFunction) lock_child_state, &val, NULL);
-      gst_iterator_free (childs);
-    }
-      break;
     default:
       break;
   }
@@ -855,7 +979,6 @@ update_start_stop_duration (GnlComposition * comp)
   GnlObject *obj;
   GnlObject *cobj = GNL_OBJECT (comp);
 
-  GST_DEBUG_OBJECT (comp, "...");
   if (!(comp->private->objects_start)) {
     GST_LOG ("no objects, resetting everything to 0");
     if (cobj->start) {
@@ -885,6 +1008,11 @@ update_start_stop_duration (GnlComposition * comp)
   if (obj->stop != cobj->stop) {
     GST_LOG_OBJECT (obj, "setting stop from %s to %" GST_TIME_FORMAT,
         GST_OBJECT_NAME (obj), GST_TIME_ARGS (obj->stop));
+    if (comp->private->defaultobject) {
+      g_object_set (comp->private->defaultobject, "duration", obj->stop, NULL);
+      g_object_set (comp->private->defaultobject, "media-duration", obj->stop, NULL);
+    }
+    comp->private->segment->stop = obj->stop;
     cobj->stop = obj->stop;
     g_object_notify (G_OBJECT (cobj), "stop");
   }
@@ -931,10 +1059,14 @@ no_more_pads_object_cb (GstElement * element, GnlComposition * comp)
       } else {
         /* toplevel element */
         gnl_composition_ghost_pad_set_target (comp, pad);
-        if (comp->private->childseek)
+        if (comp->private->childseek) {
+	  GST_INFO_OBJECT (comp, "Sending pending seek for %s",
+			   GST_OBJECT_NAME (object));
           if (!(gst_pad_send_event (pad, comp->private->childseek)))
-            GST_WARNING_OBJECT (comp, "Sending seek event failed!");
+            GST_ERROR_OBJECT (comp, "Sending seek event failed!");
+	}
         comp->private->childseek = NULL;
+	gst_pad_set_blocked_async (pad, FALSE, (GstPadBlockCallback) pad_blocked, comp);
       }
       /* remove signal handler */
       g_signal_handler_disconnect (object, entry->nomorepadshandler);
@@ -944,7 +1076,6 @@ no_more_pads_object_cb (GstElement * element, GnlComposition * comp)
 
   COMP_OBJECTS_UNLOCK (comp);
 
-  gst_object_unref (pad);
 
   GST_DEBUG_OBJECT (comp, "end");
 }
@@ -1050,7 +1181,6 @@ compare_relink_stack (GnlComposition * comp, GList * stack)
   GST_DEBUG_OBJECT (comp, "curo:%p, curn:%p", curo, curn);
 
   for (; curn; curn = g_list_next (curn)) {
-    GST_DEBUG_OBJECT (curn->data, "...");
     newobj = GNL_OBJECT (curn->data);
 
     if (pnew)
@@ -1124,17 +1254,16 @@ update_pipeline (GnlComposition * comp, GstClockTime currenttime,
 
   update_start_stop_duration (comp);
 
-  /* only modify the pipeline if we're in state != PLAYING */
   if ((GST_CLOCK_TIME_IS_VALID (currenttime))) {
     GstState state = GST_STATE (comp);
     GstState nextstate =
         (GST_STATE_NEXT (comp) ==
         GST_STATE_VOID_PENDING) ? GST_STATE (comp) : GST_STATE_NEXT (comp);
-/*     (GST_STATE_NEXT (comp) == GST_STATE_VOID_PENDING) ? GST_STATE (comp) : GST_STATE_NEXT (comp); */
     GList *stack = NULL;
     GList *deactivate;
     GstPad *pad = NULL;
     GstClockTime new_stop;
+    gboolean switchtarget = FALSE;
 
     GST_DEBUG_OBJECT (comp,
         "now really updating the pipeline, current-state:%s",
@@ -1164,7 +1293,7 @@ update_pipeline (GnlComposition * comp, GstClockTime currenttime,
         if (change_state)
           gst_element_set_state (GST_ELEMENT (deactivate->data), state);
         gst_element_set_locked_state (GST_ELEMENT (deactivate->data), TRUE);
-        deactivate = g_list_next (deactivate);
+        deactivate = g_list_delete_link (deactivate, deactivate);
       }
 
       GST_DEBUG_OBJECT (comp, "Finished de-activating objects no longer used");
@@ -1172,25 +1301,17 @@ update_pipeline (GnlComposition * comp, GstClockTime currenttime,
 
     /* if toplevel element has changed, redirect ghostpad to it */
     if ((stack) && ((!(comp->private->current))
-            || (comp->private->current->data != stack->data))) {
-      GST_DEBUG_OBJECT (comp, "Top stack object has changed, switching pad");
-      pad = get_src_pad (GST_ELEMENT (stack->data));
-      if (pad) {
-        gnl_composition_ghost_pad_set_target (comp, pad);
-        gst_object_unref (pad);
-      } else {
-        /* The pad might be created dynamically */
-      }
-    } else {
-      GST_DEBUG_OBJECT (comp,
-          "Top stack object is still the same, keeping existing pad");
-    }
+            || (comp->private->current->data != stack->data)))
+      switchtarget = TRUE;
 
     GST_DEBUG_OBJECT (comp, "activating objects in new stack to %s",
         gst_element_state_get_name (nextstate));
 
     /* activate new stack */
+    if (comp->private->current)
+      g_list_free (comp->private->current);
     comp->private->current = stack;
+
     while (stack) {
       gst_element_set_locked_state (GST_ELEMENT (stack->data), FALSE);
       if (change_state)
@@ -1203,15 +1324,48 @@ update_pipeline (GnlComposition * comp, GstClockTime currenttime,
     if (comp->private->current) {
       GstEvent *event;
 
-      event = get_new_seek_event (comp, initial);
+      COMP_OBJECTS_LOCK (comp);
 
+      event = get_new_seek_event (comp, initial);
+      
       pad = get_src_pad (GST_ELEMENT (comp->private->current->data));
 
       if (pad) {
-        if (!(gst_pad_send_event (pad, event))) {
-          GST_WARNING_OBJECT (comp, "Couldn't send seek");
-          ret = FALSE;
-        }
+	GST_DEBUG_OBJECT (comp, "We have a valid toplevel element pad %s:%s",
+			  GST_DEBUG_PAD_NAME (pad));
+
+	if (switchtarget) {
+	  GST_DEBUG_OBJECT (comp, "Top stack object has changed, switching pad");
+	  if (pad) {
+ 	    GST_LOG_OBJECT (comp, "sending seek event");
+ 	    if (!(gst_pad_send_event (pad, event)))
+ 	      ret = FALSE;
+ 	    else {
+ 	      GST_LOG_OBJECT (comp, "seek event sent successfully to %s:%s",
+ 			      GST_DEBUG_PAD_NAME (pad));
+ 	      GST_LOG_OBJECT (comp, "Setting the composition's ghostpad target to %s:%s",
+ 			      GST_DEBUG_PAD_NAME (pad));
+ 	      gnl_composition_ghost_pad_set_target (comp, pad);
+ 	    }
+	    
+	  } else {
+	    GST_LOG_OBJECT (comp, "No srcpad was available on stack's toplevel element");
+	    /* The pad might be created dynamically */
+	  }
+	} else {
+	  GST_DEBUG_OBJECT (comp,
+			    "Top stack object is still the same, keeping existing pad");
+	}
+
+        if (!switchtarget) {
+	  if (!(gst_pad_send_event (pad, event))) {
+	    GST_ERROR_OBJECT (comp, "Couldn't send seek");
+	    ret = FALSE;
+	  }
+	GST_LOG_OBJECT (comp, "seek event sent successfully to %s:%s",
+			GST_DEBUG_PAD_NAME (pad));
+	}
+	gst_pad_set_blocked_async (pad, FALSE, (GstPadBlockCallback) pad_blocked, comp);
         gst_object_unref (pad);
       } else {
         GST_DEBUG_OBJECT (comp->private->current,
@@ -1219,13 +1373,15 @@ update_pipeline (GnlComposition * comp, GstClockTime currenttime,
         comp->private->childseek = event;
         ret = TRUE;
       }
+
+    COMP_OBJECTS_UNLOCK (comp);
+
     }
   } else {
     COMP_OBJECTS_UNLOCK (comp);
   }
 
   GST_DEBUG_OBJECT (comp, "Returning %d", ret);
-
   return ret;
 }
 
@@ -1233,30 +1389,32 @@ static void
 object_start_changed (GnlObject * object, GParamSpec * arg,
     GnlComposition * comp)
 {
-  GST_DEBUG_OBJECT (object, "...");
-
   comp->private->objects_start = g_list_sort
       (comp->private->objects_start, (GCompareFunc) objects_start_compare);
 
   comp->private->objects_stop = g_list_sort
       (comp->private->objects_stop, (GCompareFunc) objects_stop_compare);
 
-  update_pipeline (comp, GST_CLOCK_TIME_NONE, FALSE, TRUE);
+  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object))
+    update_pipeline (comp, comp->private->segment_start, TRUE, TRUE);
+  else
+    update_start_stop_duration (comp);
 }
 
 static void
 object_stop_changed (GnlObject * object, GParamSpec * arg,
     GnlComposition * comp)
 {
-  GST_DEBUG_OBJECT (object, "...");
-
   comp->private->objects_stop = g_list_sort
       (comp->private->objects_stop, (GCompareFunc) objects_stop_compare);
 
   comp->private->objects_start = g_list_sort
       (comp->private->objects_start, (GCompareFunc) objects_start_compare);
 
-  update_pipeline (comp, GST_CLOCK_TIME_NONE, FALSE, TRUE);
+  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object))
+    update_pipeline (comp, comp->private->segment_start, TRUE, TRUE);
+  else
+    update_start_stop_duration (comp);
 }
 
 static void
@@ -1271,7 +1429,10 @@ object_priority_changed (GnlObject * object, GParamSpec * arg,
   comp->private->objects_stop = g_list_sort
       (comp->private->objects_stop, (GCompareFunc) objects_stop_compare);
 
-  update_pipeline (comp, GST_CLOCK_TIME_NONE, FALSE, TRUE);
+  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object))
+    update_pipeline (comp, comp->private->segment_start, TRUE, TRUE);
+  else
+    update_start_stop_duration (comp);
 }
 
 static void
@@ -1280,7 +1441,12 @@ object_active_changed (GnlObject * object, GParamSpec * arg,
 {
   GST_DEBUG_OBJECT (object, "...");
 
-  update_pipeline (comp, GST_CLOCK_TIME_NONE, FALSE, TRUE);
+  update_start_stop_duration (comp);
+
+  if (comp->private->current && OBJECT_IN_ACTIVE_SEGMENT (comp, object))
+    update_pipeline (comp, comp->private->segment_start, TRUE, TRUE);
+  else
+    update_start_stop_duration (comp);
 }
 
 static void
@@ -1297,6 +1463,15 @@ object_pad_removed (GnlObject * object, GstPad * pad, GnlComposition * comp)
   }
 }
 
+static void
+object_pad_added (GnlObject * object, GstPad * pad, GnlComposition * comp)
+{
+  GST_DEBUG_OBJECT (comp, "pad %s:%s was added, blocking it",
+		    GST_DEBUG_PAD_NAME (pad));
+
+  gst_pad_set_blocked_async (pad, TRUE, (GstPadBlockCallback) pad_blocked, comp);
+}
+
 static gboolean
 gnl_composition_add_object (GstBin * bin, GstElement * element)
 {
@@ -1316,25 +1491,49 @@ gnl_composition_add_object (GstBin * bin, GstElement * element)
 
   gst_object_ref (element);
 
+  if ((GNL_OBJECT (element)->priority == G_MAXUINT32) && comp->private->defaultobject) {
+    GST_WARNING_OBJECT (comp, "We already have a default source, remove it before adding new one");
+    ret = FALSE;
+    goto chiringuito;
+  }
+
   ret = GST_BIN_CLASS (parent_class)->add_element (bin, element);
   if (!ret)
-    goto beach;
+    goto chiringuito;
 
   /* wrap it and add it to the hash table */
   entry = g_new0 (GnlCompositionEntry, 1);
   entry->object = GNL_OBJECT (element);
-  entry->starthandler = g_signal_connect (G_OBJECT (element),
-      "notify::start", G_CALLBACK (object_start_changed), comp);
-  entry->stophandler = g_signal_connect (G_OBJECT (element),
-      "notify::stop", G_CALLBACK (object_stop_changed), comp);
-  entry->priorityhandler = g_signal_connect (G_OBJECT (element),
-      "notify::priority", G_CALLBACK (object_priority_changed), comp);
+  if ((GNL_OBJECT (element)->priority != G_MAXUINT32)) {
+    /* Only react on non-default objects properties */
+    entry->starthandler = g_signal_connect (G_OBJECT (element),
+					    "notify::start", G_CALLBACK (object_start_changed), comp);
+    entry->stophandler = g_signal_connect (G_OBJECT (element),
+					   "notify::stop", G_CALLBACK (object_stop_changed), comp);
+    entry->priorityhandler = g_signal_connect (G_OBJECT (element),
+					       "notify::priority", G_CALLBACK (object_priority_changed), comp);
+  } else {
+    /* We set the default source start/stop values to 0 and composition-stop */
+    g_object_set (element,
+		  "duration", (GstClockTimeDiff) GNL_OBJECT (comp)->stop,
+		  "media-duration", (GstClockTimeDiff) GNL_OBJECT (comp)->stop, NULL);
+    g_object_set (element, "start", 0, NULL);
+    g_object_set (element, "media-start", 0, NULL);
+  }
   entry->activehandler = g_signal_connect (G_OBJECT (element),
       "notify::active", G_CALLBACK (object_active_changed), comp);
   entry->padremovedhandler = g_signal_connect (G_OBJECT (element),
       "pad-removed", G_CALLBACK (object_pad_removed), comp);
+  entry->padaddedhandler = g_signal_connect (G_OBJECT (element),
+					     "pad-added", G_CALLBACK (object_pad_added), comp);
   g_hash_table_insert (comp->private->objects_hash, element, entry);
 
+  /* Special case for default source */
+  if (GNL_OBJECT(element)->priority == G_MAXUINT32) {
+    comp->private->defaultobject = GNL_OBJECT (element);
+    goto chiringuito;
+  }
+
   /* add it sorted to the objects list */
   comp->private->objects_start = g_list_append
       (comp->private->objects_start, element);
@@ -1362,22 +1561,32 @@ gnl_composition_add_object (GstBin * bin, GstElement * element)
         GST_TIME_ARGS (GNL_OBJECT (comp->private->objects_stop->data)->start),
         GST_TIME_ARGS (GNL_OBJECT (comp->private->objects_stop->data)->stop));
 
-  /* update pipeline */
-  COMP_OBJECTS_UNLOCK (comp);
-  update_pipeline (comp, GST_CLOCK_TIME_NONE, FALSE, TRUE);
-  COMP_OBJECTS_LOCK (comp);
+  GST_DEBUG_OBJECT (comp, "segment_start:%"GST_TIME_FORMAT" segment_stop:%"GST_TIME_FORMAT,
+		    GST_TIME_ARGS (comp->private->segment_start),
+		    GST_TIME_ARGS (comp->private->segment_stop));
 
+  COMP_OBJECTS_UNLOCK (comp);
+
+  /* If we added within currently configured segment, update pipeline */
+  if (OBJECT_IN_ACTIVE_SEGMENT (comp, element))
+    update_pipeline (comp, comp->private->segment_start, TRUE, TRUE);
+  else
+    update_start_stop_duration (comp);
+  
 beach:
   gst_object_unref (element);
-  COMP_OBJECTS_UNLOCK (comp);
   return ret;
+
+ chiringuito:
+  COMP_OBJECTS_UNLOCK (comp);
+  goto beach;
 }
 
 
 static gboolean
 gnl_composition_remove_object (GstBin * bin, GstElement * element)
 {
-  gboolean ret;
+  gboolean ret = GST_STATE_CHANGE_FAILURE;
   GnlComposition *comp = GNL_COMPOSITION (bin);
 
   GST_DEBUG_OBJECT (bin, "element %s", GST_OBJECT_NAME (element));
@@ -1390,30 +1599,42 @@ gnl_composition_remove_object (GstBin * bin, GstElement * element)
 
   gst_element_set_locked_state (element, FALSE);
 
-  ret = GST_BIN_CLASS (parent_class)->remove_element (bin, element);
-  if (!ret)
-    goto beach;
+  /* handle default source */
+  if (GNL_OBJECT(element)->priority == G_MAXUINT32) {
+    comp->private->defaultobject = NULL;
+  } else {
+    
+    /* remove it from the objects list and resort the lists */
+    comp->private->objects_start = g_list_remove
+      (comp->private->objects_start, element);
+    comp->private->objects_start = g_list_sort
+      (comp->private->objects_start, (GCompareFunc) objects_start_compare);
+    
+    comp->private->objects_stop = g_list_remove
+      (comp->private->objects_stop, element);
+    comp->private->objects_stop = g_list_sort
+      (comp->private->objects_stop, (GCompareFunc) objects_stop_compare);
+  }
 
   if (!(g_hash_table_remove (comp->private->objects_hash, element)))
-    goto beach;
-
-  /* remove it from the objects list and resort the lists */
-  comp->private->objects_start = g_list_remove
-      (comp->private->objects_start, element);
-  comp->private->objects_start = g_list_sort
-      (comp->private->objects_start, (GCompareFunc) objects_start_compare);
-
-  comp->private->objects_stop = g_list_remove
-      (comp->private->objects_stop, element);
-  comp->private->objects_stop = g_list_sort
-      (comp->private->objects_stop, (GCompareFunc) objects_stop_compare);
+    goto chiringuito;
 
   COMP_OBJECTS_UNLOCK (comp);
-  update_pipeline (comp, GST_CLOCK_TIME_NONE, FALSE, TRUE);
-  COMP_OBJECTS_LOCK (comp);
+
+  /* If we removed within currently configured segment, or it was the default source, *
+   * update pipeline */
+  if (OBJECT_IN_ACTIVE_SEGMENT (comp, element) || (GNL_OBJECT(element)->priority == G_MAXUINT32))
+    update_pipeline (comp, comp->private->segment_start, TRUE, TRUE);
+  else
+    update_start_stop_duration (comp);
+
+  ret = GST_BIN_CLASS (parent_class)->remove_element (bin, element);
 
 beach:
   gst_object_unref (element);
-  COMP_OBJECTS_UNLOCK (comp);
   return ret;
+
+ chiringuito:
+  COMP_OBJECTS_UNLOCK (comp);
+  goto beach;
 }
