@@ -37,8 +37,6 @@ struct _GnlPadPrivate
   GstPadLinkFunction linkfunc;
   GstPadUnlinkFunction unlinkfunc;
 
-  gboolean flush_hack;
-  gboolean need_flush;
 };
 
 GST_BOILERPLATE (GnlObject, gnl_object, GstBin, GST_TYPE_BIN);
@@ -542,6 +540,8 @@ internalpad_event_function (GstPad * internal, GstEvent * event)
 {
   GnlPadPrivate *priv = gst_pad_get_element_private (internal);
   GnlObject *object = priv->object;
+  GstMessage * message = NULL;
+  gboolean res;
 
   GST_DEBUG_OBJECT (internal, "event:%s", GST_EVENT_TYPE_NAME (event));
 
@@ -555,20 +555,9 @@ internalpad_event_function (GstPad * internal, GstEvent * event)
     case GST_PAD_SRC:
       if (GST_EVENT_TYPE (event) == GST_EVENT_NEWSEGMENT) {
         event = translate_outgoing_new_segment (object, event);
-      } else if (priv->ghostpriv->flush_hack && !priv->ghostpriv->need_flush) {
-        /* FIXME : REMOVE THIS AFTER FLUSH HACK SOLVED */
-        GST_DEBUG_OBJECT (internal, "Flush hack in effect");
-        if ((GST_EVENT_TYPE (event) == GST_EVENT_FLUSH_START)
-            || (GST_EVENT_TYPE (event) == GST_EVENT_FLUSH_STOP)) {
-          GST_DEBUG_OBJECT (object,
-              "dropping flush event because of flush hack");
-          gst_event_unref (event);
-          return TRUE;
-        }
-      } else {
-        GST_DEBUG_OBJECT (internal,
-            "didn't drop flush ghost->flush_hack:%d ->need_flush:%d",
-            priv->ghostpriv->flush_hack, priv->ghostpriv->need_flush);
+	message = gst_message_new_segment_start (GST_OBJECT (object),
+						 GST_FORMAT_TIME,
+						 (gint64) object->start);
       }
 
       break;
@@ -581,7 +570,11 @@ internalpad_event_function (GstPad * internal, GstEvent * event)
       break;
   }
   GST_DEBUG_OBJECT (internal, "Calling priv->eventfunc %p", priv->eventfunc);
-  return priv->eventfunc (internal, event);
+  res = priv->eventfunc (internal, event);
+  if (message)
+    gst_element_post_message (GST_ELEMENT (object), message);
+
+  return res;
 }
 
 /*
@@ -658,31 +651,6 @@ internalpad_unlink_function (GstPad * internal)
         "priv->unlinkfunc == NULL !! What's going on ?");
 }
 
-/* Add flush flag to seek event */
-static GstEvent *
-flush_hack_check (GstEvent * event, GnlPadPrivate * priv)
-{
-  gdouble rate;
-  GstFormat format;
-  GstSeekFlags flags;
-  GstSeekType cur_type, stop_type;
-  gint64 cur, stop;
-
-  gst_event_parse_seek (event, &rate, &format, &flags,
-      &cur_type, &cur, &stop_type, &stop);
-
-  if ((flags & GST_SEEK_FLAG_FLUSH)) {
-    GST_DEBUG ("Already has FLUSH flag");
-    priv->need_flush = TRUE;
-    return event;
-  } else {
-    GST_DEBUG ("Creating new event with flush flag");
-    gst_event_unref (event);
-    priv->need_flush = FALSE;
-    return gst_event_new_seek (rate, format,
-        flags | GST_SEEK_FLAG_FLUSH, cur_type, cur, stop_type, stop);
-  }
-}
 
 static gboolean
 ghostpad_event_function (GstPad * ghostpad, GstEvent * event)
@@ -701,9 +669,6 @@ ghostpad_event_function (GstPad * ghostpad, GstEvent * event)
   switch (priv->dir) {
     case GST_PAD_SRC:
       if (GST_EVENT_TYPE (event) == GST_EVENT_SEEK) {
-        /* FIXME: REMOVE THIS AFTER FLUSH HACK SOLVED */
-        if (priv->flush_hack)
-          event = flush_hack_check (event, priv);
 
         event = translate_incoming_seek (object, event);
       }
@@ -786,8 +751,6 @@ control_internal_pad (GstPad * ghostpad, GnlObject * object)
   priv->object = object;
   priv->ghostpriv = privghost;
   priv->dir = GST_PAD_DIRECTION (ghostpad);
-  priv->flush_hack = privghost->flush_hack;
-  priv->need_flush = privghost->need_flush;
   gst_object_unref (internal);
 }
 
@@ -807,7 +770,6 @@ ghostpad_link_function (GstPad * ghostpad, GstPad * peer)
   GST_DEBUG_OBJECT (ghostpad,
       "linking went ok, getting internal pad and overriding query/event functions");
 
-  control_internal_pad (ghostpad, GNL_OBJECT (GST_PAD_PARENT (ghostpad)));
 
   return ret;
 
@@ -858,7 +820,6 @@ gnl_object_ghost_pad_full (GnlObject * object, const gchar * name,
     GstPad * target, gboolean flush_hack)
 {
   GstPadDirection dir = GST_PAD_DIRECTION (target);
-  GnlPadPrivate *priv;
   GstPad *ghost;
 
   GST_DEBUG_OBJECT (object, "name:%s, target:%p, flush_hack:%d",
@@ -876,16 +837,13 @@ gnl_object_ghost_pad_full (GnlObject * object, const gchar * name,
     return NULL;
   }
 
-  /* FIXME : REMOVE THIS ONCE THE FLUSH HACK HAS GONE !! */
-  priv = gst_pad_get_element_private (ghost);
-  priv->flush_hack = flush_hack;
-  priv->need_flush = TRUE;
 
-  /* add it to element */
+ /* add it to element */
   if (!(gst_element_add_pad (GST_ELEMENT (object), ghost))) {
     GST_WARNING ("couldn't add newly created ghostpad");
     return NULL;
   }
+  control_internal_pad (ghost, object);
 
   return ghost;
 }
@@ -919,8 +877,6 @@ gnl_object_ghost_pad_no_target (GnlObject * object, const gchar * name,
   priv = g_new0 (GnlPadPrivate, 1);
   priv->dir = dir;
   priv->object = object;
-  priv->need_flush = TRUE;
-  priv->flush_hack = FALSE;
   gst_pad_set_element_private (ghost, priv);
 
   return ghost;
@@ -973,11 +929,8 @@ gnl_object_ghost_pad_set_target (GnlObject * object, GstPad * ghost,
   gst_pad_set_query_function (ghost,
       GST_DEBUG_FUNCPTR (ghostpad_query_function));
 
-  /* maybe the ghostpad is already linked */
-  if (GST_PAD_IS_LINKED (ghost)) {
-    GST_LOG_OBJECT (ghost, "ghostpad was already linked");
+  if (!GST_OBJECT_IS_FLOATING (ghost))
     control_internal_pad (ghost, object);
-  }
 
   return TRUE;
 }
@@ -1248,3 +1201,4 @@ gnl_object_change_state (GstElement * element, GstStateChange transition)
 beach:
   return ret;
 }
+
