@@ -80,7 +80,7 @@ sinkpad_event_probe (GstPad * sinkpad, GstEvent * event, CollectStructure * coll
     segment = (Segment *) collect->expected_segments->data;
 
     if (compare_segments (segment, event)) {
-      collect->expected_segments = g_list_next (collect->expected_segments);
+      collect->expected_segments = g_list_remove (collect->expected_segments, segment);
       g_free (segment);
     }
     collect->gotsegment = TRUE;
@@ -423,10 +423,13 @@ GST_START_TEST (test_one_after_other)
 
   fail_if (gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE);
 
+  gst_object_unref (GST_OBJECT(sinkpad));
   ASSERT_OBJECT_REFCOUNT_BETWEEN(pipeline, "main pipeline", 1, 2);
   gst_object_unref (pipeline);
   ASSERT_OBJECT_REFCOUNT_BETWEEN(bus, "main bus", 1, 2);
   gst_object_unref (bus);
+
+  g_free (collect);
 }
 
 GST_END_TEST;
@@ -547,10 +550,13 @@ GST_START_TEST (test_one_under_another)
   fail_if (gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE);
 
   gst_bus_poll (bus, GST_MESSAGE_ANY, GST_SECOND / 2);
+  gst_object_unref (GST_OBJECT (sinkpad));
   ASSERT_OBJECT_REFCOUNT_BETWEEN(pipeline, "main pipeline", 1, 2);
   gst_object_unref (pipeline);
   ASSERT_OBJECT_REFCOUNT_BETWEEN(bus, "main bus", 1, 2);
   gst_object_unref (bus);
+
+  g_free (collect);
 }
 
 GST_END_TEST;
@@ -574,7 +580,7 @@ GST_START_TEST (test_one_bin_after_other)
   /*
     Source 1
     Start : 0s
-    Duration : 2s
+    Duration : 1s
     Priority : 1
   */
   source1 = videotest_in_bin_gnl_src ("source1", 0, 1 * GST_SECOND, 1, 1);
@@ -583,8 +589,8 @@ GST_START_TEST (test_one_bin_after_other)
 
   /*
     Source 2
-    Start : 2s
-    Duration : 2s
+    Start : 1s
+    Duration : 1s
     Priority : 1
   */
   source2 = videotest_in_bin_gnl_src ("source2", 1 * GST_SECOND, 1 * GST_SECOND, 2, 1);
@@ -730,10 +736,13 @@ GST_START_TEST (test_one_bin_after_other)
 
   fail_if (gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE);
 
+  gst_object_unref (GST_OBJECT (sinkpad));
   ASSERT_OBJECT_REFCOUNT_BETWEEN(pipeline, "main pipeline", 1, 2);
   gst_object_unref (pipeline);
   ASSERT_OBJECT_REFCOUNT_BETWEEN(bus, "main bus", 1, 2);
   gst_object_unref (bus);
+
+  g_free (collect);
 }
 
 GST_END_TEST;
